﻿using System;

namespace SimpleDataStructures
{
    /// <summary>
    /// This class implements a simple list that uses array to store its elements.
    /// </summary>
    /// <typeparam name="T">Type of elements stored in the array.</typeparam>
    class SimpleList<T>
    {
        private T[] elements;
        protected int usedSlots;

        /// <summary>
        /// Initializes a new instance of <see cref="SimpleList{T}"/>.
        /// </summary>
        /// <param name="initialSize">Initial size of the array in which elements are stored.</param>
        public SimpleList(int initialSize)
        {
            this.elements = new T[initialSize];
            this.usedSlots = 0;
        }
        /// <summary>
        /// Initializes a new instance of <see cref="SimpleList{T}"/>.
        /// </summary>
        public SimpleList()
            : this(1)
        {
        }

        /// <summary>
        /// Gets or sets the number of elements the list can hold without resizing.
        /// </summary>
        public int Capacity
        {
            get { return this.elements.Length; }
            set { ExtendArraySize(value); }
        }

        /// <summary>
        /// Resizes the array that contains the elements of the list to the size which is twice the current size.
        /// </summary>
        private void ExtendArraySize()
        {
            ExtendArraySize(this.elements.Length * 2);
        }

        /// <summary>
        /// Resizes the array that contains the elements of the list to <paramref name="newSize"/>.
        /// If <paramref name="newsize"/> is lower than the current size, the array remains intact.
        /// </summary>
        /// <param name="newSize">New size of the array.</param>
        private void ExtendArraySize(int newSize)
        {
            int size = Math.Max(newSize, elements.Length);
            T[] biggerArray = new T[size];
            this.elements.CopyTo(biggerArray, 0);
            this.elements = biggerArray;
        }

        /// <summary>
        /// Inserts <paramref name="elements"/> to the position <paramref name="index"/>.
        /// </summary>
        /// <param name="elements">Elements to be inserted</param>
        /// <param name="index">Position of the element</param>
        /// <exception cref="ArgumentOutOfRangeException">Thrown when
        /// <paramref name="index"/> is bigger than the number of elements in the array.</exception>
        public void Insert(int index, params T[] elements)
        {
            for(int i = 0; i < elements.Length; i++)
            {
                InsertOneElement(index + i, elements[i]);
            }
        }
        
        
        protected void InsertOneElement(int index, T element)
        {
            if (index > this.usedSlots)
                throw new ArgumentOutOfRangeException();

            //if the array is too small, create a new, bigger one
            if (this.usedSlots == this.elements.Length)
                ExtendArraySize();

            for (int i = usedSlots - 1; i >= index; i--)
            {
                this.elements[i + 1] = this.elements[i];
            }
            this.elements[index] = element;
            usedSlots++;
        }

        /// <summary>
        /// Removes the element at the position <paramref name="index"/>.
        /// </summary>
        /// <param name="index">position of the element to be removed.</param>
        /// <returns>Returns the removed element.</returns>
        /// <exception cref="ArgumentOutOfRangeException">Thrown when <paramref name="index"/>
        /// </exception>
        public T Remove(int index)
        {
            if (index >= usedSlots)
                throw new IndexOutOfRangeException();
            else
            {
                var removedElement = this.elements[index];
                for (int i = index; i < usedSlots - 1; i++)
                {
                    this.elements[i] = this.elements[i+1];
                }
                this.elements[usedSlots - 1] = default(T);
                usedSlots--;
                return removedElement;
            }
        }

        public override string ToString()
        {
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            for (int i = 0; i < usedSlots; i++)
            {
                sb.Append(this.elements[i].ToString());
                sb.Append(' ');
            }
            return sb.ToString();
        }
    }
}